# YASARA MOVIE
# TOPIC:    3. Molecular Dynamics
# TITLE:    3.1. The theory behind molecular dynamics simulations
# REQUIRES: Dynamics
# AUTHOR:   Elmar Krieger
# LICENSE:  GPL

if Help
  # If we are showing a help movie, wait for clicks on the ContinueButton
  Button='ContinueButton'
else
  # Otherwise just wait for a click on the left mouse button
  Button='LeftButton'

# Change working directory to the place of this macro
# (so that we don't have to specify paths when loading stuff).
CD (MacroDir)
RequireVersion 3.11.16
OnError Continue
UserInput Off
Console Off

Boundary Wall
Longrange None

#Menu Off

#
# SLIDE 1: Title Page
#
TitlePage:
# Left handed coordinate system
Clear
CoordSys Left
Projection perspective
HideMessage
LightSource 45,45,Ambience=85,Shadow=65
AtomSize 100

# Don't show console and head up display
HUD Off
MarkAtom None
StickRadius 70
BallStickRadius 50,70

Style Ball
FontFog Off
ColorBG Blue,White
# The rotating protein
LoadPDB gprotein
PosObj gprotein,14.7,-11,17
StickObj gprotein
ColorObj gprotein,Cyan
AutoRotateObj gprotein,Y=-0.1
# The title
MakeTextObj Title,Width=800,Height=80
Font Name=Baikal,Height=40%,Color=White,Spacing=2,Depth=100,DepthCol=ff8000
# NOTE: Wrong colors in MacOSX 10.4 are a graphics driver problem (shaders affected only)
PosText 50%,50%,Justify=Center
Print "Molecular Dynamics Simulations\n"
Print "Watching Nature@Work"
PosObj Title,0,190,730
MakeTextObj Author,Width=20,Height=1
Font Name=Baikal,Height=100%,Color=Black,Depth=0
PosText 50%,100%
Print "by\n"
Print "Elmar.Krieger@yasara.org"
PosObj Author,0,-3,18.9
# Load Logos
LoadPNG logos
ShowImage logos,0,0,1024,768,Alpha=100,Priority=1
if Help
  ShowMessage 'Hi (FirstName)! Welcome to a 15-minute YASARA Movie about the theory of molecular dynamics simulations.'
Wait 1
# Take snapshot here, Eliza!
# MovieSnapshot
Wait (Button)
if Help
  ShowMessage "Imagine you had a microscope with almost infinite magnification, like those guys on starship Voyager..."
AutoMoveObj gprotein,X=0.1
Wait 20
AutoRotateObj Author,X=-5
AutoMoveObj Author,Y=-0.05,Z=-0.1
Wait 20
# Look through 'o'
AutoPosObj Title,-204.7,52.4,50.5,Steps=300
DelObj Author
# Show ProteinG again
AutoMoveObj gprotein,X=0
AutoRotateObj gprotein,Y=0
OriObj gprotein,0,0,0
ColorObj gprotein,Element
BallObj gprotein
PosObj gprotein,33,0,278
Wait (Button)
if Help
  ShowMessage "This would allow you to look at a protein, down to the scale of individual atoms..."
AutoPosObj gprotein,X=0,Steps=150
Wait (Button)
if Help
  ShowMessage "You could zoom in as closely as you want..."
AutoPosObj gprotein,Z=108,Steps=200
Cell 50,50,50
PosObj SimCELL,0,0,108
CleanObj gprotein
# Start Simulation
ForceField Amber96
Cutoff 5.24
TempCtrl Rescale
Temp 25C
Sim On
Wait (Button)
if Help
  ShowMessage "..and watch the atoms as they tumble through space and time, gaining knowledge about protein structure and dynamics."
AutoPos Z=12.8,Steps=600
# MovieSnapshot
# Fly away
Wait (Button)
if Help
  ShowMessage "That is the basic idea behind molecular dynamics simulations: Use the computer as a microscope to look into a virtual world."
Sim Off
AutoPos Z=350,Steps=100,Wait=No
Wait 60
AutoRotate Y=3
Wait 60
AutoPos Z=40,Steps=30
AutoRotate Y=-0.1
AutoMove Y=0.1,Z=1
Wait 200
AutoMove Y=2,Z=1
Wait 200

#
# SLIDE 2: Simulation of air
#
TheBasics:
Clear
LoadSce air2
Rotate Y=12
Style Ball
ColorBG c000c0,Cyan
LightSource Ambience=0,Shadow=50
SwitchObj 1-4,Off
obj=1
for text in "Let's start simple. Imagine you are looking at a box filled with hot air. There's 21% oxygen, the molecules shown in red...",
            "Then there's 79% of nitrogen, shown in blue...",
            "There is also a little bit of something else, so we add three molecules of carbon dioxide just for the fun...",
            "And finally, of course the box. The box is very important in molecular dynamics, because it confines the simulated system..."
  if Help
    ShowMessage (text)
  Wait (Button)
  SwitchObj (obj),On
  obj=obj+1
CombineObj 2 3,1
if Help
  ShowMessage "So how can we simulate this very simple system with the computer?"
Wait (Button)
if Help
  ShowMessage "Even though our box with air is simple, it's already too large for a complicated quantum chemical approach."
  Wait (Button)
  ShowMessage "So we must stick to a simple 'classical approximation' which can nevertheless include terms of non-classical origin."
# MovieSnapshot
AutoPos Z=-70,Steps=300
AutoPos -15,-15,25,Steps=240
Wait (Button)
if Help
  ShowMessage "'Classical approximation' mainly refers to using the famous equation of Newtonian dynamics: Force = mass x Acceleration"
# F=m*a
MakeTextObj Fma,Width=40,Height=4
Font Name=Arial,Height=100%,Color=White,Depth=20,DepthCol=ffc000
PosText 50%,0%,Justify=Center
Print "F = m * A"
MakeTextObj aFom,Width=40,Height=4
PosText 50%,0%,Justify=Center
Print "A = F / m"
PosObj Fma,0,50,30
PosObj aFom,0,-50,30
AutoPosObj Fma,Y=5,Steps=100
# MovieSnapshot
Wait (Button)
if Help
  ShowMessage "If we shuffle that around a bit, we can express the Acceleration A as a function of Force and mass (Uppercase means a vector)."
AutoPosObj aFom,Y=-5,Steps=100
Wait (Button)
if Help
  ShowMessage "As we know the mass of every atom, all we need to get the Acceleration is - the Force, which comes from a 'force field'."
AutoRotateObj Fma,X=1
AutoPosObj Fma,Y=25,Steps=100,Wait=No
AutoPosObj aFom,Y=0,Steps=100
DelObj Fma
Wait (Button)
if Help
  ShowMessage "A force field is an equation which expresses the force acting on an atom as a function of the positions of the other atoms."
# MovieSnapshot
AutoRotateObj aFom,X=1
AutoPosObj aFom,Y=-25,Steps=100
DelObj aFom
Wait (Button)

# Intramolecular interactions
if Help
  ShowMessage "A force field equation consists of many components: Intramolecular forces to describe chemical bonds (usually simple 'springs'),"
MakeTextObj Intra,Width=12,Height=18
Font Name=Times,Height=4%,Color=White,Depth=0
PosObj Intra,Y=-4.5,Z=0.1
PosText 50%,90%,Justify=Center
Print "Intramolecular Forces\n"
Font Height=3%
PosText 10%,80%,Justify=Left
Print "Bonds, "
BallStickRes Atom 604
ShowArrow Start=StepFromAtom,604,Y=1.5,End=StepFromAtom,605,Y=1.5,Radius=0.2,Color=Grey
Wait (Button)

if Help
  ShowMessage "bond angles (the forces that keep the angle between two chemical bonds at its equilibrium value and involve three atoms),"
Print "Angles, "
ShowArrow Start=StepFromAtom,604,Y=-1.5,End=StepFromAtom,606,Y=-1.5,Radius=0.2,Color=Grey
Wait (Button)

if Help
  ShowMessage "dihedral angles (the forces that favour the staggered conformation of ethane over the eclipsed one and involve four atoms)."
Print "Dihedrals"
Wait (Button)
# MovieSnapshot
AutoRotateObj Intra,X=2
Wait 90
DelObj Intra

# Intermolecular interactions
if Help
  ShowMessage "And there are intermolecular forces on all atoms that belong to different molecules or are separated by more than three bonds."
MakeTextObj Inter,Width=12,Height=18
Font Height=4%
PosText 50%,90%,Justify=Center
Print "Intermolecular Forces\n"
Font Height=3%
Print "Van der Waals, Electrostatic"
PosObj Inter,Y=-4.5,Z=0.1
Wait (Button)

if Help
  ShowMessage "To save processor time, intermolecular forces are usually only calculated for atoms close by, not all atoms in the box."
for atom in 139,117,287,122,32,522,278,428
  ShowArrow Start=AtAtom,604,End=AtAtom,(atom),Radius=0.2,Color=Green
  Wait (Button)
# MovieSnapshot
AutoRotateObj Inter,X=2
Wait 90
DelObj Inter
AutoRotate Y=0.1
AutoMove Z=0.5
Wait 300

#
# SLIDE 3: Bonds + Angles
#
Bonds:
Clear
ColorBG Blue,White
LightSource Ambience=85,Shadow=65
Style Ball
LoadPDB co2
Cell Auto
SwitchObj SimCELL,off
ForceField Nova
Sim On
TempCtrl SteepDes
PullForce 128
FixAtom 2-3
ColAtom 2-3,Element
MarkAtom 1
ColorObj co2,Element
ShowArrow Start=StepFromAtom,1,End=StepFromAtom,2,Radius=0.2,Color=Grey
Pos Y=-3.3,Z=2.1
# Text
LoadPNG bonds,TransCol=Magenta
ShowImage bonds,0,0,1024,768,Alpha=0.0,Priority=1
if Help
  ShowMessage "Let's look at one carbon dioxide molecule.The arrow symbolizes a chemical bond.Press <Ctrl> and the right mouse button to pull."
Wait (Button)
if Help
  ShowMessage "The force acting on the marked atom is a function of the distance r, the equilibrium bond length r0 and the force constant k.."
AutoMoveImage bonds, 0,0, 1024,768, Alpha=100, Steps=60
# MovieSnapshot
Wait (Button)

# The mathematics of bonds (harmonic potential)
BondMath:
Clear
ColorBG Blue,White
LoadPNG bonds2,TransCol=Magenta
LoadPNG paper
ShowImage bonds2,0,0,1024,768,Alpha=0.0,Priority=1
AutoMoveImage bonds2, 0,0, 1024,768, Alpha=100, Steps=60
Wait 60
ShowImage paper,0,-768,1024,768,Alpha=100,Priority=1
pos=-122,-220,-390,-474,-592,-768
text="Let's have a closer look at the mathematics. We want to calculate the force f acting on our atom due to a chemical bond...",
     "The current distance r between the two bonded atoms is just the length of vector R, which links the atoms in 3D space.",
     "The vector R therefore has three components, one along each axis: dx, dy and dz.",
     "The distance r equals the length of vector R, which is the square root of (dx*dx + dy*dy + dz*dz).",
     "The force vector F equals the scalar force f times the unit vector R/r (to indicate the direction of the force).",
     "So that's the formula to calculate the force vector F acting on our atom due to a bond with equilibrium length r0 and strength k.."
for i=1 to 6
  AutoMoveImage paper,0,(pos(i)),1024,768,Alpha=100,Steps=60
  Wait 60
  if Help
    ShowMessage (text(i))
  Wait (Button)
if Help
  # Help message covers the last formula, hide it
  HideMessage
  Wait (Button)
# MovieSnapshot

#
# Dihedrals
#
Dihedrals:
Clear
Style Stick
LoadPDB ethane
PosObj ethane,Y=-1.3,Z=3
Cell Auto
SwitchObj SimCELL,off
ForceField Nova
PullForce 16
TempCtrl Anneal
ShowArrow Start=AtAtom,3,End=AtAtom,8,Radius=0.2,Color=Grey
if Help
  ShowMessage "Now the dihedral angles. The repulsive force acting between atoms separated by 3 bonds favours the staggered conformation."
Wait (Button)
if Help
  ShowMessage "You can see that the three hydrogen atoms on one side are placed as far away as possible from those on the other side."
AutoRotate Y=1
Wait 450
AutoRotate Y=0
MakeTextObj Dihedral,Width=12,Height=18
Font Name=Times,Height=4%,Color=White,Depth=0
PosObj Dihedral,Y=-4.5,Z=0.1
PosText 50%,90%,Justify=Center
Print "Dihedral Forces\n"
Sim On
FixAtom 5-8
ColorObj ethane,Element
Wait (Button)
if Help
  ShowMessage "If you pull the marked atom (<Ctrl>+<RightButton>), it will always find its way back to the staggered conformation."
MarkAtom 2
PullForce 2
Wait (Button)
# MovieSnapshot
HideMessage
AutoRotateObj Dihedral,X=2
Wait 90

#
# Improper Dihedrals
#
Impropers:
Clear
LoadSce tyrosine
Style Stick
Cell Auto
SwitchObj SimCELL,off
ForceField Nova
PullForce 16
TempCtrl Anneal
if Help
  ShowMessage "Look at this tyrosine. It contains a planar ring, and the forces keeping the ring planar are called 'improper dihedrals'."
Wait (Button)
if Help
  ShowMessage "'Improper dihedrals' are a special form of dihedral angles. The associated forces involve a central and three bound atoms."
AutoRotate Y=1
Wait 360
AutoRotate Y=0
MakeTextObj Improper,Width=12,Height=18
Font Name=Times,Height=4%,Color=White,Depth=0
PosObj Improper,Y=-4.5,Z=0.1
PosText 50%,90%,Justify=Center
Print "Improper Dihedral Forces\n"
Sim On
FixObj 1crn
FreeAtom 13-23
ColorObj 1crn,Element
Wait (Button)
if Help
  ShowMessage "If you pull the marked atom (<Ctrl>+<RightButton>), you will see that the entire ring stays planar."
MarkAtom 13
PullForce 2
Wait (Button)
HideMessage
# MovieSnapshot
AutoRotateObj Improper,X=2
Wait 90

#
# Lennard Jones Forces
#
LennardJones:
Clear
LoadSce oxygen
Style Ball,Ball
SwitchObj SimCELL,off
ForceField Nova
Cutoff 10
Temp 10K
TempCtrl Anneal
MakeTextObj vdw,Width=12,Height=18
Font Name=Times,Height=4%,Color=White,Depth=0
PosObj vdw,Y=-4.5,Z=0.1
PosText 50%,90%,Justify=Center
Print "Van der Waals Forces\n"
# Text
LoadPNG vdw,TransCol=Magenta
ShowImage vdw,0,0,1024,768,Alpha=0.0,Priority=1
if Help
  ShowMessage "Correlated motions of electrons in neighboring atoms create temporary dipoles of the same orientation - the atoms attract."
Wait (Button)
if Help
  ShowMessage "This attraction is called a 'Van der Waals' force. In the force field, it is approximated with a 'Lennard-Jones' function."
AutoMoveImage vdw, 0,0, 1024,768, Alpha=100, Steps=60
Wait 60
Wait (Button)
if Help
  ShowMessage "You can see how the two oxygen molecules attract each other. This takes a long time, the Van der Waals forces are weak."
SimSteps 20
Sim On
Wait (Button)
HideMessage
# MovieSnapshot
AutoRotateObj vdw,X=2
Wait 90

#
# Electrostatic Forces
#
Electrostatics:
Clear
LoadSce hoh
Style Ball,Ball
SwitchObj SimCELL,off
ForceField Amber96
Cutoff 10
TempCtrl Anneal
MakeTextObj electro,Width=12,Height=18
Font Name=Times,Height=4%,Color=White,Depth=0
PosObj electro,Y=-4.5,Z=0.1
PosText 50%,90%,Justify=Center
Print "Electrostatic Forces\n"
# Text
LoadPNG coulomb,TransCol=Magenta
ShowImage coulomb,0,0,1024,768,Alpha=0.0,Priority=1
if Help
  ShowMessage "A very important part of the force field are the electrostatic forces (=Coulomb forces) between charged atoms."
Wait (Button)
if Help
  ShowMessage "Water molecules are neutral, but the electronegative oxygen pulls charge away from the hydrogens, creating partial charges."
AutoMoveImage coulomb, 0,0, 1024,768, Alpha=100, Steps=60
Wait 60
Wait (Button)
if Help
  ShowMessage "The resulting strong electrostatic forces are responsible for a very important feature of proteins: the hydrogen bonds."
SimSteps 3
Sim On
Wait (Button)
HideMessage
# MovieSnapshot
AutoRotateObj electro,X=2
Wait 90

#
# The 8 steps to Molecular Dynamics
#
Integration:
Clear
LoadSce air2
Pos Y=-180
MakeTextObj aFom,Width=40,Height=4
Font Name=Arial,Height=100%,Color=White,Depth=20,DepthCol=ffc000
PosText 50%,0%,Justify=Center
Print "A = F / m"
PosObj aFom,0,-50,30
AutoPosObj aFom,Y=0,Steps=100
LoadPNG steps8,TransCol=Magenta
LoadPNG paper
ShowImage steps8,0,0,1024,768,Alpha=0.0,Priority=1
if Help
  ShowMessage "And we are back at our initial formula. Thanks to the force field, we now know the force F, so we can get the acceleration A."
Wait (Button)
AutoPosObj aFom,Y=-50,Steps=100
AutoMoveImage steps8, 0,0, 1024,768, Alpha=100, Steps=60
Wait 60
ShowImage paper,0,-768,1024,768,Alpha=100,Priority=1

pos=-200,-264,-328,-390,-450,-510,-570,-630,-700
text="How does it all fit together? What do we need to get the atoms moving? Here are the 8 steps...",
     "Our atom has a certain velocity V. When we start a simulation, this velocity is randomly chosen to match a given temperature.",
     "Then we sum up the forces acting on the atom using our force field equation. This gives us the force vector F.",
     "With the central equation of Newtonian dynamics, we calculate the acceleration vector A from the force F and the atom mass.",
     "Next we choose a small timestep dt for our simulation.This is typically 1-2 femtoseconds.Too large values crash the simulation.",
     "Knowing the acceleration A and dt, we can calculate the change of atom velocity dV during the time interval dt.",
     "This immediately gives us the new atom velocity.",
     "Which in turn allows to calculate the atom movement during the time interval dt, V*dt, and thus new atom coordinates.",
     "So that's it, we go back to step 2 and start another simulation cycle. The math name for this procedure is 'numerical integration'."
for i=1 to 9
  AutoMoveImage paper,0,(pos(i)),1024,768,Alpha=100,Steps=60
  Wait 60
  if Help
    ShowMessage (text(i))
  Wait (Button)
# MovieSnapshot

# Show the air again
if Help
  ShowMessage "Finally, after this long tour through the secrets of molecular dynamics, we can simulate our box with hot air..."
DelObj aFom
AutoPos Y=0,Steps=100
AutoMoveImage steps8, 0,0, 1024,768, Alpha=0.0, Steps=60
Wait 100
DelImage steps8
ForceField Nova
Cutoff 5
SimSteps 4
Temp 600K
TimeStep 2,4
TempCtrl Rescale
Wait (Button)
if Help
  ShowMessage "Here we go, air at 600 Kelvin..."
Sim On
AutoPos Z=100,Steps=600
Wait (Button)
HideMessage

#
# Applications of molecular dynamics in an ideal world
#
ApplicationsIdeal:
Clear
LoadPNG applications1
LoadPNG paper
ShowImage applications1,0,0,1024,768,Alpha=100,Priority=1
Wait 30
ShowImage paper,0,-768,1024,768,Alpha=100,Priority=1

#pos=-330,-390,-450,-510,-570,-768
pos=-330,-400,-470,-540,-610,-768
text="The headline says 'ideal world', because many of the things listed do not work as well as we'd like them to...",
     "In principle, you can fold proteins and predict conformational changes just by simulating a peptide chain.",
     "You can also analyze the effects of mutations on protein stability,",
     "refine structures by starting a simulation from an approximate model,",
     "dock proteins and ligands by 'shooting' them toward each other from random initial orientations,",
     "and design drugs by running simulations of different ligands and checking which one binds strongest."
for i=1 to 6
  AutoMoveImage paper,0,(pos(i)),1024,768,Alpha=100,Steps=60
  Wait 60
  if Help
    ShowMessage (text(i))
  Wait (Button)
# MovieSnapshot

#
# The problem of MD
#
Problems:
Clear
LoadSce 1dv8
DelRes HOH
LoadPNG fig1a
LoadPNG fig1b
Style Ribbon
ShowAtom Sidechain CA
AutoPos Y=2,Steps=300
ForceField Amber96
TimeStep 2,1
Cutoff 5.24
Temp 25C
TempCtrl Rescale
SimSteps 1
if Help
  ShowMessage "Now we take the step from the simple simulation of air to a protein. This is PDB entry 1dv8, a carbohydrate binding protein."
Wait (Button)
if Help
  ShowMessage "Normally you run a simulation in a box filled with water, which has been omitted here so that you can see the protein..."
Sim On
Wait (Button)
ShowImage fig1a, -172,-124, 1024,768, Alpha=0.0, Priority=1
AutoMoveImage fig1a, -172,-124, 1024,768, Alpha=100, Steps=20
if Help
  ShowMessage "Imagine you built models by homology and want to minimize them. This plot shows you the change in RMSD(from the target)."
Wait (Button)
ShowImage fig1b, -172,-124, 1024,768, Alpha=100, Priority=1
AnimateImage fig1b, BlendLeft, Rest=100000, BlendRight, Steps=100
Wait 100
if Help
  ShowMessage "As you can see, the RMSD goes up, the models move in the wrong direction, due to the limited force field accuracy...."
Wait (Button)
if Help
  ShowMessage "What can we do about that? How can we make force fields more accurate without losing performance?"
#Question mark
Sim Pause
LoadPNG question,TransCol=Magenta
DelImage fig1a
ShowImage question,-2048,-1536, 5120,3840, Alpha=0.5, Priority=1
AutoMoveImage question,0,0, 1024,768, Alpha=100, Steps=60, 1
Wait 60
Wait (Button)
DelImage fig1b
DelImage question

#
# Force Field Optimization
#
text="One solution to the problem is a 'self-parameterizing force field'. Simply change one force field parameter randomly...",
     "Then use this new force field to energy minimize models and known structures.",
     "See if the models got any better (lower RMSD to the known targets) and how much the structures got worse...",
     "If the new force field is better than the last one, accept it and change the next parameter randomly...",
     "Otherwise reject it and go back to the previous force field. Now try to change another parameter and continue the game..."
for i=1 to 5
  LoadPNG nova(i+5)
  ShowImage nova(i+5), 0,0, 1024,768, Alpha=100, Priority=1
  if Help
    ShowMessage (text(i))
  Wait (Button)
  if i==5 and Help
    HideMessage
    Wait (Button)
  DelImage nova(i+5)

"""
#
# Applications of molecular dynamics in the real world
#
ApplicationsReal:
Clear
LoadPNG applications2
LoadPNG paper
ShowImage applications2,0,0,1024,768,Alpha=100,Priority=1
Wait 30
ShowImage paper,0,-768,1024,768,Alpha=100,Priority=1

#pos=-330,-390,-450,-510,-570,-768
pos=-330,-330,-330,-400,-470,-540,-610,-768
text="Having mentioned the difficulties and limitations of MD, there are still lots of useful applications in drug design.",
     "If there are less than 6 degrees of freedom (e.g. docking an alpha helix into the major DNA groove)...",
     "...one can run a few thousand MD simulations of the docking with different initial orientations to sample conformational space.",
     "MD also helps a lot to handle receptor flexibility: one obtains an ensemble of receptor structures and docks against all of them.",
     "Many docking methods return complex structures with lots of bumps and other errors. MD/EM help with the fine-tuning.",
     "If one runs a simulation of the complex and it falls apart, that's a noteworthy prediction about its stability.",
     "Sometimes one can obtain binding free energies for small ligands using 'FEP', an exotic method that involves adjusting...",
     "...force field parameters during the simulation. Unfortunately many people have reported random results, so this is not widely used."
for i=1 to count pos
  AutoMoveImage paper,0,(pos(i)),1024,768,Alpha=100,Steps=60
  Wait 60
  if Help
    ShowMessage (text(i))
  Wait (Button)
HideMessage
Wait (Button)
"""

#
# End screen
#
LoadPNG exitscr
Brake 5000
Sim Continue
Temp 25C
TempCtrl Rescale
ShowImage exitscr, 0,0, 1024,768, Alpha=0.0, Priority=1
AutoMoveImage exitscr, 0,0, 1024,768, Alpha=80, Steps=20
Wait 20
Wait (Button)
Stop

